@@ -1,5 +1,5 @@
 use webcore::value::{Reference, Value};
-use webcore::try_from::TryInto;
+use webcore::try_from::{TryInto, TryFrom};
 use webapi::event_target::{IEventTarget, EventTarget};
 use webapi::node::{INode, Node};
 use webapi::element::Element;
@@ -9,7 +9,7 @@ use webapi::text_node::TextNode;
 use webapi::location::Location;
 use webapi::parent_node::IParentNode;
 use webapi::non_element_parent_node::INonElementParentNode;
-use private::TODO;
+use webapi::dom_exception::{InvalidCharacterError, NamespaceError};
 
 /// The `Document` interface represents any web page loaded in the browser and
 /// serves as an entry point into the web page's content, which is the DOM tree.
@@ -21,6 +21,12 @@ use private::TODO;
 #[reference(subclass_of(EventTarget, Node))]
 pub struct Document( Reference );
 
+error_enum_boilerplate! {
+    CreateElementNsError,
+    InvalidCharacterError,
+    NamespaceError
+}
+
 impl IEventTarget for Document {}
 impl IParentNode for Document {}
 impl INode for Document {}
@@ -52,10 +58,19 @@ impl Document {
     ///
     /// [(JavaScript docs)](https://developer.mozilla.org/en-US/docs/Web/API/Document/createElement)
     // https://dom.spec.whatwg.org/#ref-for-dom-document-createelement
-    pub fn create_element( &self, tag: &str ) -> Result< Element, TODO > {
-        unsafe {
-            Ok( js!( return @{self}.createElement( @{tag} ); ).into_reference_unchecked().unwrap() )
-        }
+    pub fn create_element( &self, tag: &str ) -> Result< Element, InvalidCharacterError > {
+        js_try!( return @{self}.createElement( @{tag} ); ).unwrap()
+    }
+
+    /// Creates an element with the specified namespace URI and qualified name.
+    /// To create an element without specifying a namespace URI, use the `createElement` method.
+    ///
+    /// [(JavaScript docs)](https://developer.mozilla.org/en-US/docs/Web/API/Document/createElementNS)
+    // https://dom.spec.whatwg.org/#ref-for-dom-document-createelementns
+    pub fn create_element_ns( &self, namespace_uri: &str, tag: &str ) -> Result< Element, CreateElementNsError > {
+        js_try!(
+            return @{self}.createElementNS( @{namespace_uri}, @{tag} );
+        ).unwrap()
     }
 
     /// Creates a new text node.
@@ -167,3 +182,33 @@ impl Document {
         );
     }
 }
+
+
+#[cfg(all(test, feature = "web_test"))]
+mod web_tests {
+    use super::*;
+
+    #[test]
+    fn test_create_element_invalid_character() {
+        match document().create_element("-invalid tag") {
+            Err(InvalidCharacterError{..}) => (),
+            v => panic!("expected InvalidCharacterError, got {:?}", v),
+        }
+    }
+
+    #[test]
+    fn test_create_element_ns_invalid_character() {
+        match document().create_element_ns("", "-invalid tag") {
+            Err(CreateElementNsError::InvalidCharacterError(_)) => (),
+            v => panic!("expected InvalidCharacterError, got {:?}", v),
+        }
+    }
+
+    #[test]
+    fn test_create_element_ns_namespace_error() {
+        match document().create_element_ns("", "illegal_prefix:svg") {
+            Err(CreateElementNsError::NamespaceError(_)) => (),
+            v => panic!("expected NamespaceError, got {:?}", v),
+        }
+    }
+}
\ No newline at end of file
