@@ -38,6 +38,9 @@ pub mod console;
 pub mod html_collection;
 pub mod child_node;
 pub mod gamepad;
+pub mod selection;
+#[cfg(feature = "experimental_features_which_may_break_on_minor_version_bumps")]
+pub mod midi;
 
 #[cfg(feature = "futures-support")]
 pub mod timer_future;
